# frozen_string_literal: true

# Implement FEP-fffd Proxy Objects for Distributed Press by creating new
# activities and actors under supported protocols/schemes.
#
# Priority is low so it's the last thing we do.  If you need to modify
# the original activities after this plugins runs, use priority 0.
#
# The hook runs on pre render so we can add the extra pages.
Jekyll::Hooks.register :site, :pre_render, priority: 1 do |site|
  require 'jekyll/activity_pub/link'

  # Changes the URLs from a base URL to have a different scheme
  #
  # @param object [Any]
  # @param base_url [String]
  # @param scheme [String]
  def convert_uris(object, base_url, scheme)
    case object
    when Hash
      object.transform_values do |value|
        convert_uris(value, base_url, scheme)
      end
    when Array
      object.map do |value|
        convert_uris(value, base_url, scheme)
      end
    when String
      if object.start_with? base_url
        Addressable::URI.parse(object).tap do |uri|
          uri.scheme = scheme
        end.to_s
      else
        object
      end
    when Nokogiri::HTML5::DocumentFragment
      object.dup.tap do |html|
        html.css('[src]').each do |element|
          element['src'] = convert_uris(element['src'], base_url, scheme)
        end
      end
    when Jekyll::ActivityPub::Helper
      convert_uris(object.data, base_url, scheme)
    else
      object
    end
  end

  site.pages.each do |page|
    case page
    when Jekyll::ActivityPub::Activity, Jekyll::ActivityPub::Actor
      site = page.site
      uri = Addressable::URI.parse page.data['id']
      url = [ Jekyll::ActivityPub::Link.new(site, page.data['url'], 'canonical', 'text/html') ]

      %w[https ipns hyper bittorrent].each do |protocol|
        uri.scheme = protocol
        url <<
          Jekyll::ActivityPub::Link.new(site, uri.to_s, 'alternate', 'application/ld+json; profile="https://www.w3.org/ns/activitystreams"')
        url <<
          Jekyll::ActivityPub::Link.new(site, uri.to_s, 'alternate', 'application/activity+json')
      end

      %w[ipns hyper bittorrent].each do |protocol|
        json = convert_uris(page.data, site.config['url'], protocol).reject do |_, v|
          v.nil? || (v.respond_to?(:empty?) && v.empty?)
        end

        uri  = Addressable::URI.parse(json['id'])
        uri.path = uri.path.sub('.jsonld', ".#{protocol}.jsonld")
        json['id'] = uri.to_s
        path = uri.path.sub(%r{\A/}, '')

        Jekyll::PageWithoutAFile.new(site, site.source, File.dirname(path), File.basename(path)).tap do |alternate|
          site.pages << alternate

          alternate.content = json.to_json
        end
      end
    end
  end
end
